#!/usr/bin/env python3

import bisect
import sys

# Leading comment of the input file
leading_comment = ""
# This dictionary will map each start point to its corresponding end point and
# InCB property
indic_conjunct_break_properties = {}
# This list will hold the starting code points of each range for binary search
indic_conjunct_break_start_points = []


def parse_indic_conjunct_break_line(line):
    # Skip comments and empty lines
    if line.startswith('#') or not line.strip() or '; InCB;' not in line:
        return None

    # Parse the valid line to extract the range and the InCB property
    range_part, _, prop = line.split(';')
    prop = prop.split('#')[0].strip()  # Clean up the InCB property string

    # Handle single code points and ranges
    if '..' in range_part:
        start_str, end_str = range_part.split('..')
        start = int(start_str, 16)
        end = int(end_str, 16)
    else:
        start = end = int(range_part.strip(), 16)

    return start, end, prop


def load_indic_conjunct_break_data(file_path):
    global leading_comment
    global indic_conjunct_break_properties
    global indic_conjunct_break_start_points

    with open(file_path, 'r', encoding='utf-8') as input_file:
        leading_comment = input_file.readline()
        leading_comment += input_file.readline()
        for line in input_file:
            parsed_data = parse_indic_conjunct_break_line(line)
            if parsed_data:
                start, end, prop = parsed_data
                indic_conjunct_break_properties[start] = (end, prop)

    # Sort by the start points
    indic_conjunct_break_properties = {
        k: v for k, v in
        sorted(indic_conjunct_break_properties.items(),
               key=lambda item: item[0])
    }
    indic_conjunct_break_start_points = list(
        indic_conjunct_break_properties.keys())


def output_indic_conjunct_break_data():
    print('/* The content of this file is generated from:')
    print(leading_comment, end='')
    print('*/')
    print('')
    print('#include "indicconjunctbreakdef.h"')
    print('')
    print('static const struct IndicConjunctBreakProperties incb_prop[] = {')
    for start, (end, prop) in indic_conjunct_break_properties.items():
        print(f"    {{0x{start:04X}, 0x{end:04X}, InCB_{prop}}},")
    print('};')


def get_indic_conjunct_break(char):
    if not indic_conjunct_break_start_points:
        load_indic_conjunct_break_data('DerivedCoreProperties.txt')
    # Find the code point of the character
    code_point = ord(char)

    # Binary search for the closest start point less than or equal to the
    # code point
    index = bisect.bisect_right(indic_conjunct_break_start_points,
                                code_point) - 1
    if index >= 0:
        start = indic_conjunct_break_start_points[index]
        end, prop = indic_conjunct_break_properties[start]
        if start <= code_point <= end:
            return prop

    # If not found, return None or a default value if you have one
    return None


def main():
    input_file_path = sys.argv[1] if sys.argv[1:] else \
        'DerivedCoreProperties.txt'
    load_indic_conjunct_break_data(input_file_path)
    output_indic_conjunct_break_data()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import unicode_data_property


def output_east_asian_width_data(east_asian_width_properties, skip=''):
    print('static const struct EastAsianWidthProperties eaw_prop[] = {')
    for start, (end, prop) in east_asian_width_properties.items():
        if prop == skip:
            continue
        print(f"    {{0x{start:04X}, 0x{end:04X}, EAW_{prop}}},")
    print('};')


def lookup_east_asian_width(cp, east_asian_width_properties):
    for start, (end, prop) in east_asian_width_properties.items():
        if cp >= start and cp <= end:
            return prop


def output_is_op_east_asian_function(line_break_properties,
                                     east_asian_width_properties):
    print('bool ub_is_op_east_asian(utf32_t ch) {')
    print('    return false', end='')
    last_east_asian_op = None
    for start, (end, prop) in line_break_properties.items():
        if prop == 'OP' or prop == 'CP':
            for ch in range(start, end + 1):
                w = lookup_east_asian_width(ch, east_asian_width_properties)
                if w in ('F', 'W', 'H'):
                    # As of Unicode 15.1, there is no east asian CP. If this
                    # doesn't hold in a future version, we need to adjust code.
                    assert prop == 'OP'

                    if not last_east_asian_op:
                        print(f'\n        || (ch >= 0x{ch:04X}', end='')
                        last_east_asian_op = ch
                else:
                    if last_east_asian_op:
                        print(f' && ch <= 0x{ch:04X})', end='')
                        last_east_asian_op = None
    if last_east_asian_op:
        print(')', end='')
    print(';')
    print('}')


def main():
    input_file_path = sys.argv[1] if sys.argv[1:] else \
        'EastAsianWidth.txt'
    line_break_file = sys.argv[2] if len(sys.argv) > 2 else 'LineBreak.txt'
    leading_comment, east_asian_width_properties = \
        unicode_data_property.load_data(input_file_path)
    leading_comment_2, line_break_properties = \
        unicode_data_property.load_data(line_break_file)

    print('/* The content of this file is generated from:')
    print(leading_comment, end='')
    print(leading_comment_2, end='')
    print('*/')
    print()
    print('#include "eastasianwidthdef.h"')
    print()
    output_east_asian_width_data(east_asian_width_properties, skip='N')
    print()
    output_is_op_east_asian_function(line_break_properties,
                                     east_asian_width_properties)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from __future__ import print_function
import sys

ranges = []
last_first = None
last_second = None
last_value = None
for line in sys.stdin:
    first, second = line.rstrip().split('..')
    second, value = second.split(';')
    value = value.lstrip()
    first = int(first, 16)
    second = int(second, 16)
    if last_value is None:
        last_first = first
        last_second = second
        last_value = value
    elif last_second + 1 != first or last_value != value:
        ranges.append((last_first, last_second, last_value))
        last_first = first
        last_second = second
        last_value = value
    else:
        last_second = second

ranges.append((last_first, last_second, last_value))

# print ranges for supplementary planes

BMP = 65536

sp = [r for r in ranges if r[0] >= BMP]

print("""*/

#include "linebreakdef.h"

/** Line breaking properties for supplementary planes */
const struct LineBreakProperties lb_prop_supplementary[] = {""")

for first, second, value in sp:
    if first >= BMP:
        print("\t{{ 0x{:X}, 0x{:X}, LBP_{} }},".format(first, second, value))

print("""\t{ 0xFFFFFFFF, 0xFFFFFFFF, LBP_Undefined }
};

const unsigned int lb_prop_supplementary_len =
    sizeof(lb_prop_supplementary) / sizeof(lb_prop_supplementary[0]);
""")

# print BMP table

print("""/** Line breaking properties for BMP */
const char lb_prop_bmp[] = {""")

bmp = ['XX'] * BMP
for first, second, value in ranges:
    if first >= BMP:
        break
    bmp[first:second + 1] = [value] * (second - first + 1)


def chunks(arr, n):
    for i in range(0, len(arr), n):
        yield i, arr[i:i + n]


for i, chunk in chunks(bmp, 8):
    line = ', '.join(['LBP_{}'.format(value) for value in chunk])
    print('\t', line, sep='', end=',\n')
print("};")
